#
# Copyright 2010 Andrey Churin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import sys
import re
import string

from paste.script.command import BadCommand, Command

MODULE_NAME_REGEX = re.compile(r'^(?:[a-z][a-z0-9]*\.)*[a-z][a-z0-9]*$')
NAMESPACE_REGEX = re.compile(r'^(?:[_a-zA-Z][_a-zA-Z0-9]*\.)*[a-zA-Z][_a-zA-Z0-9]*$')
CAMELCASE_REGEX = re.compile(r'_([a-zA-Z])')

def validate_name(name, word = None):
    if not name or not MODULE_NAME_REGEX.match(name):
        raise Error('"%s" is not valid namespace'%name, word)


def validate_namespace(name, word = None):
    if not name or not NAMESPACE_REGEX.match(name):
        raise Error('"%s" is not valid namespace'%name, word)

def camel_case(name):
    CAMELCASE_REGEX.sub(lambda x: x.group(1).upper(), name)

def squeeze(value, width = 80, padding = 0):
    result = [value[:width]]
    while len(result[-1]) == width:
        value = value[width:]
        result.append(value[:width])
    return ' '*padding + ('\n' + ' '*padding).join(result)

def get_indent(s):
    "Gets string indentation"
    return len(s) - len(s.lstrip())

class ClosureCommand(Command):
    def command(self):
        try:
            self.command_execute()
        except Exception, e:
            if isinstance(e, Error):
                msg = str(e)
            else:
                msg = str(sys.exc_info()[1])
            raise BadCommand('\nAn error occurred:\n%s\n' % msg)

class Error(Exception):
    def __init__(self, msg, pos = None):
        super(Error, self).__init__(msg)
        self._pos = pos
        self._traceback = []

    def traceback(self, msg):
        self._traceback.insert(0, msg)
        return self

    def get_traceback(self):
        lines = '\n'.join(self._traceback).split('\n')
        if lines and self._pos is not None:
            line_string = lines[-1]
            pos = get_indent(line_string) + self._pos
            lines.append(' '*pos + '^')

        return '  ' + '\n  '.join(lines)

    def __str__(self):
        return self.get_traceback() + '\n' + super(Error, self).__str__()
